package com.sainyu.beidou3.demo;

import com.yang.protocolstack.ProtocolStack;
import com.yang.protocolstack.protocol.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.net.InetAddress;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;

public class SocketFrame extends JFrame {

    /**
     * 程序界面宽度
     */
    private static final int WIDTH = 500;

    /**
     * 程序界面高度
     */
    private static final int HEIGHT = 500;

    //数据显示界面
    private JTextArea dataView = new JTextArea();
    private JScrollPane scrollDataView = new JScrollPane(dataView);

    // 网设置面板
    private JPanel socketPanel = new JPanel();
    private JLabel ipLabel = new JLabel("IP");
    private JLabel portLabel = new JLabel("端口号");
    private JLabel protocolTypeLabel = new JLabel("协议");
    private JTextField ipField = new JTextField(12);
    private JTextField portField = new JTextField(12);
    private JComboBox protocolTypeChoice = new JComboBox();

    private JButton LiuLiangCheck = new JButton("流量信息");
    private JButton LiuLiangReSet = new JButton("记录清零");

    // 操作面板
    private JPanel operatePanel = new JPanel();
    private JButton serialPortOperate = new JButton("打开网口");
    private JButton iCInfoOperate = new JButton("读卡");
    private JButton iCSubInfoOperate = new JButton("读取下属");
    private JButton locateOperate = new JButton("定位");
    private JButton powerOperate = new JButton("功率检测");
    private JButton stateCheck = new JButton("自检信息");

    private JButton locRepOperate = new JButton("位置报告");
    private JButton sendMsgCOperate = new JButton("通信（汉字）");
    private JButton sendMsgBOperate = new JButton("通信（BCD）");
    private JButton sendMsgMOperate = new JButton("通信（混发）");

    private int id;
    private int SubCount = 0 ;
    private int protocolType;
    private SocketClient socketClient;
    private ProtocolStack protocolStack;

    private Thread readThread = new Thread(new Runnable() {
        @Override
        public void run() {
            while (true) {
                Response response;
                if (protocolStack != null) {
                    response = protocolStack.getResponse();
                    if (response != null) {
                        switch (response.getMsgType()) {
                            case MsgType.BSI_PROTOCOL:
                                //2.1协议
                                dataView.append( "波束强度：" + Arrays.toString(((BDBSIResponse) response).getPower()) + "\r\n");
                                break;
                            case MsgType.ZDA_PROTOCOL:
                                //2.1协议
                                dataView.append("时间：" + ((BDZDAResponse) response).getTimeBJ() + "\r\n");
                                break;
                            case MsgType.GLZK_PROTOCOL:
                                //4.0协议
                                dataView.append("波束强度：" + Arrays.toString(((BDGLZKResponse) response).getPower())+ "\r\n");
                                break;
                            case MsgType.ICXX_PROTOCOL:
                                dataView.append("北斗卡号：" + ((BDICXXResponse) response).getId() + "\r\n" +
                                        "通播地址：" + ((BDICXXResponse) response).getBroadcastId() + "\r\n" +
                                        "用户机特征：" + ((BDICXXResponse) response).getFeature() + "\r\n" +
                                        "服务频度：" + ((BDICXXResponse) response).getServerFreq() + "\r\n" +
                                        "通信长度：" + ((BDICXXResponse) response).getMaxCommLength() + "\r\n");
                                id = ((BDICXXResponse) response).getId();
                                SubCount = ((BDICXXResponse) response).getNumber();
                                break;
                            case MsgType.ICI_PROTOCOL:
                                dataView.append("北斗卡号：" + ((BDICIResponse) response).getLocalId() + "\r\n" +
                                        "通播地址：" + ((BDICIResponse) response).getBroadcastId() + "\r\n" +
                                        "用户机特征：" + ((BDICIResponse) response).getFeature() + "\r\n" +
                                        "服务频度：" + ((BDICIResponse) response).getServerFreq() + "\r\n" +
                                        "通信长度：" + ((BDICIResponse) response).getMaxCommLength() + "\r\n");
                                id = ((BDICIResponse) response).getLocalId();
                                break;
                            case MsgType.SJXX_PROTOCOL:
                                //4.0协议
                                dataView.append("时间：" + ((BDSJXXResponse) response).getTime() + "\r\n");
                                break;
                            case MsgType.DWR_PROTOCOL:
                                dataView.append(String.format("定位位置：经度： %.06f 纬度： %.06f %n",((BDDWRResponse) response).getLongitude(),((BDDWRResponse) response).getLatitude()));
                                break;
                            case MsgType.DWXX_PROTOCOL:
                                dataView.append(String.format("定位位置：经度： %s 纬度： %s %n",((BDDWXXResponse) response).getLongitude(),((BDDWXXResponse) response).getLatitude()));
                                break;
                            case MsgType.FKI_PROTOCOL:
                                if (((BDFKIResponse) response).isCmdRes()) {
                                    dataView.append("申请成功，等待反馈！\r\n");
                                } else {
                                    dataView.append(String.format("申请失败，频度未到。等待时间：%d s%n",((BDFKIResponse) response).getWaitTime()));
                                }
                                break;
                            case MsgType.FKXX_PROTOCOL:
                                if (((BDFKXXResponse) response).isCmdResult()) {
                                    dataView.append("申请成功，等待反馈！\r\n");
                                } else {
                                    dataView.append(String.format("申请失败，频度未到。等待时间：%d s%n",((BDFKXXResponse) response).getWaitTime()));
                                }
                                break;
                            case MsgType.WAA_PROTOCOL:
                                dataView.append(String.format("收到位置报告：经度： %.06f 纬度： %.06f %n",((BDWAAResponse) response).getLongitude(),((BDWAAResponse) response).getLatitude()));
                                break;
                            case MsgType.TXR_PROTOCOL:
                                dataView.append(String.format("收到通信信息：内容： %s 发送方： %d 通信类型： %d %n",((BDTXRResponse) response).getCommInfo(),((BDTXRResponse) response).getSenderID(), ((BDTXRResponse) response).getCodeType()));
                                break;
                            case MsgType.TXXX_PROTOCOL:
                                if (((BDTXXXResponse) response).getType() == 3) {
                                    dataView.append(String.format("4.0协议通过通信发送接收位置报告，接收到的位置信息: 时间： %tT，经度： %s，纬度： %s，高程： %d%n",new Date(((BDTXXXResponse) response).getTime()),
                                            ((BDTXXXResponse) response).getLongitude(),((BDTXXXResponse) response).getLatitude(),((BDTXXXResponse) response).getAltitude()));
                                } else {
                                    dataView.append(String.format("收到通信信息：内容： %s 发送方： %s 通信类型： %d %n",((BDTXXXResponse) response).getContent(),((BDTXXXResponse) response).getSendId(), ((BDTXXXResponse) response).getType()));
                                }
                                break;
                            case MsgType.ZJX_PROTOCOL:{//2.1协议
                                dataView.append("自检信息 IC状态:"+ ((BDZJXResponse) response).getCardState() + "\r\n");
                                dataView.append("自检信息 硬件状态:"+ ((BDZJXResponse) response).getHardState() + "\r\n");
                                dataView.append("自检信息 入站状态:"+ ((BDZJXResponse) response).getInState() + "\r\n");
                            }
                            break ;
                            case MsgType.ZJXX_PROTOCOL:{//4.0协议
                                dataView.append("自检信息 IC状态:"+ ((BDZJXXResponse) response).getCardState() + "\r\n");
                                dataView.append("自检信息 硬件状态:"+ ((BDZJXXResponse) response).getHardState() + "\r\n");
                                dataView.append("自检信息 入站状态:"+ ((BDZJXXResponse) response).getInState() + "\r\n");
                            }
                            break;
                        }
                        dataView.setCaretPosition(dataView.getDocument().getLength());
                    } else {
                        try {
                            Thread.sleep(1);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                    }
                } else {
                    try {
                        Thread.sleep(1);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    });


    public SocketFrame() {
        initView();
        initComponents();
        actionListener();
        initData();
    }

    private void initView() {
        // 关闭程序
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        // 禁止窗口最大化
        setResizable(false);

        // 设置程序窗口居中显示
        Point p = GraphicsEnvironment.getLocalGraphicsEnvironment()
                .getCenterPoint();
        setBounds(p.x - WIDTH / 2, p.y - HEIGHT / 2, WIDTH, HEIGHT);
        this.setLayout(null);

        setTitle("网口北斗通信Demo");
    }

    private void initComponents() {
        // 数据显示
        dataView.setFocusable(false);
        scrollDataView.setBounds(10, 10, 475, 190);
        add(scrollDataView);

        // 网口设置
        socketPanel.setBorder(BorderFactory.createTitledBorder("网口设置"));
        socketPanel.setBounds(10, 200, 170, 190);
        socketPanel.setLayout(null);
        add(socketPanel);

        ipLabel.setForeground(Color.gray);
        ipLabel.setBounds(10, 30, 40, 20);
        socketPanel.add(ipLabel);

        ipField.setBounds(60, 30, 100, 20);
        socketPanel.add(ipField);

        portLabel.setForeground(Color.gray);
        portLabel.setBounds(10, 65, 40, 20);
        socketPanel.add(portLabel);

        portField.setBounds(60, 65, 100, 20);
        socketPanel.add(portField);

        protocolTypeLabel.setForeground(Color.gray);
        protocolTypeLabel.setBounds(10, 100, 40, 20);
        socketPanel.add(protocolTypeLabel);

        protocolTypeChoice.setFocusable(false);
        protocolTypeChoice.setBounds(60, 100, 100, 20);
        socketPanel.add(protocolTypeChoice);

        LiuLiangCheck.setFocusable(false);
        LiuLiangCheck.setBounds(10, 130, 100, 20);
        socketPanel.add(LiuLiangCheck);

        LiuLiangReSet.setFocusable(false);
        LiuLiangReSet.setBounds(10, 160, 100, 20);
        socketPanel.add(LiuLiangReSet);

        // 操作
        operatePanel.setBorder(BorderFactory.createTitledBorder("操作"));
        operatePanel.setBounds(200, 200, 285, 180);
        operatePanel.setLayout(null);
        add(operatePanel);


        serialPortOperate.setFocusable(false);
        serialPortOperate.setBounds(17, 22, 120, 20);
        operatePanel.add(serialPortOperate);

        iCInfoOperate.setFocusable(false);
        iCInfoOperate.setBounds(17, 52, 120, 20);
        operatePanel.add(iCInfoOperate);

        locateOperate.setFocusable(false);
        locateOperate.setBounds(17, 82, 120, 20);
        operatePanel.add(locateOperate);

        powerOperate.setFocusable(false);
        powerOperate.setBounds(17, 112, 120, 20);
        operatePanel.add(powerOperate);

        locRepOperate.setFocusable(false);
        locRepOperate.setBounds(147, 22, 120, 20);
        operatePanel.add(locRepOperate);

        sendMsgCOperate.setFocusable(false);
        sendMsgCOperate.setBounds(147, 52, 120, 20);
        operatePanel.add(sendMsgCOperate);

        sendMsgBOperate.setFocusable(false);
        sendMsgBOperate.setBounds(147, 82, 120, 20);
        operatePanel.add(sendMsgBOperate);

        sendMsgMOperate.setFocusable(false);
        sendMsgMOperate.setBounds(147, 112, 120, 20);
        operatePanel.add(sendMsgMOperate);

        iCSubInfoOperate.setFocusable(false);
        iCSubInfoOperate.setBounds(147, 142, 120, 20);
        operatePanel.add(iCSubInfoOperate);

        stateCheck.setFocusable(false);
        stateCheck.setBounds(17, 142, 120, 20);
        operatePanel.add(stateCheck);
    }

    private void actionListener() {

        LiuLiangCheck.addActionListener(e -> {
            if (protocolStack != null) {
                long[] arr =protocolStack.GetLiuLiangXX();

                Date recordDate = new Date(arr[0]);
                SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                dataView.append("记录时间：" + format.format(recordDate) + "  发送总数：" + arr[1] + "  成功：" + arr[2] + "  接收： " + arr[3] + "\r\n");
            }
        });

        LiuLiangReSet.addActionListener(e -> {
            if (protocolStack != null) {
                protocolStack.ResetLiuLiangXX();
                dataView.append("流量记录清除\r\n");
            }
        });

        serialPortOperate.addActionListener(e -> {
            if (("打开网口").equals(serialPortOperate.getText())) {
                openSocket(e);
            } else {
                closeSocket(e);
            }
        });
        iCInfoOperate.addActionListener(e -> {
            if (protocolStack != null) {
                dataView.append("读卡申请中、、" + "\r\n");
                socketClient.send(protocolStack.getDKSQ());
            }
        });

        iCSubInfoOperate.addActionListener(e -> {
            if (protocolStack != null) {
                dataView.append("读子用户申请中、、" + "\r\n");
                byte icNo = 1;
                for (int i= 0; i < SubCount/20; i++)
                    socketClient.send(protocolStack.getZYHDKSQ(icNo,id+"",false));//默认百用户  千用户时需要设置为true
            }
        });

        locateOperate.addActionListener(e -> {
            if (protocolStack != null) {
                if (id == 0) {
                    dataView.append("未读卡，在获取到卡号之后才能定位" + "\r\n");
                } else {
                    dataView.append("定位申请中、、" + "\r\n");
                    socketClient.send(protocolStack.getDWSQ(String.valueOf(id),false,0));
                }
            }
        });

        powerOperate.addActionListener(e -> {
            if (protocolStack != null) {
                dataView.append("检测功率中、、 " + "\r\n");
                socketClient.send( protocolStack.getGLSQ());
            }
        });

        stateCheck.addActionListener(e -> {
            if (protocolStack != null) {
                dataView.append("检测功率中、、 " + "\r\n");
                socketClient.send( protocolStack.getZJSQ());
            }
        });

        locRepOperate.addActionListener(e -> {
            if (protocolStack != null) {
                if (id == 0) {
                    dataView.append("未读卡，未获取到卡号" + "\r\n");
                } else {
                    dataView.append("发送位置报告中、、" + "\r\n");
                    if (protocolType == 0) {
                        //仅2.1协议
                        socketClient.send(protocolStack.getWZBG(String.valueOf(id),0,null));
                    } else {
                        socketClient.send(protocolStack.getWZXX40(String.valueOf(id),new Date().getTime(),38.123d,112.456d,153,String.valueOf(id)));
                    }
                }
            }
        });

        sendMsgBOperate.addActionListener(e -> {
            if (protocolStack != null) {
                if (id == 0) {
                    dataView.append("未读卡，未获取到卡号" + "\r\n");
                } else {
                    dataView.append("以代码方式发送通信信息，内容为 123 " + "\r\n");
                    socketClient.send(protocolStack.getTXSQ(1, String.valueOf(id),"123",true, String.valueOf(id)));
                }
            }
        });

        sendMsgCOperate.addActionListener(e -> {
            if (protocolStack != null) {
                if (id == 0) {
                    dataView.append("未读卡，未获取到卡号" + "\r\n");
                } else {
                    dataView.append("以汉字方式发送通信信息，内容为 汉字 " + "\r\n");
                    socketClient.send(protocolStack.getTXSQ(0, String.valueOf(id),"汉字",true, String.valueOf(id)));
                }
            }
        });

        sendMsgMOperate.addActionListener(e -> {

            socketClient.send(protocolStack.getTXSQ(2, String.valueOf("459650"),"汉字BCD",true, String.valueOf("459651")));
            return ;

//            if (protocolStack != null) {
//                if (id == 0) {
//                    dataView.append("未读卡，未获取到卡号" + "\r\n");
//                } else {
//                    dataView.append("以混发方式发送通信信息，内容为 汉字BCD " + "\r\n");
//                    socketClient.send(protocolStack.getTXSQ(2, String.valueOf(id),"汉字BCD",true, String.valueOf(id)));
//                }
//            }
        });

    }

    private void closeSocket(ActionEvent e) {
        protocolStack.stop();
        socketClient.stop();
        dataView.append("网口已关闭" + "\r\n");
        serialPortOperate.setText("打开网口");
        protocolStack = null;
    }

    private void openSocket(ActionEvent e) {
        String ip = ipField.getText();
        String prot = portField.getText();
        protocolType = 0;
        protocolType = protocolTypeChoice.getSelectedIndex();
        try {
            socketClient = new SocketClient(ip, Integer.parseInt(prot)) {
                @Override
                public void onReceive(InetAddress addr, byte[] bytes) {
                    protocolStack.setBytes(bytes);
                }

                @Override
                public void onDisconnect(InetAddress addr) {
                    dataView.setText("网口已关闭" + "\r\n");
                    serialPortOperate.setText("打开网口");
                }
            };
            protocolStack = new ProtocolStack();
            protocolStack.init(protocolType,false);
            socketClient.start();
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        dataView.setText("网口已打开" + "\r\n");
        serialPortOperate.setText("关闭网口");
        if (!readThread.isAlive()) {
            readThread.start();
        }
    }

    @SuppressWarnings("unchecked")
    private void initData() {
        protocolTypeChoice.addItem("2.1协议");
        protocolTypeChoice.addItem("4.0协议");

    }

    public static void main(String args[]) {
        EventQueue.invokeLater(() -> new SocketFrame().setVisible(true));
    }

}
